@@ -7,15 +7,21 @@ compare.default <- function(object, ...)
 
 
 ###-----compare.synds------------------------------------------------------
-compare.synds <- function(object, data, vars = NULL, 
-  msel = NULL, breaks = 20, nrow = 2, ncol = 2, rel.size.x = 1,
-  cols = c("#1A3C5A","#4187BF"), stat = "percents", ...){ 
+compare.synds <- function(object, data, vars = NULL, msel = NULL,  
+                          stat = "percents", breaks = 20, 
+                          nrow = 2, ncol = 2, rel.size.x = 1,
+                          utility.stats = c("pMSE", "S_pMSE", "df"),
+                          cols = c("#1A3C5A","#4187BF"),  
+                          plot = TRUE, table = FALSE, ...){    
                                                                          
- if (is.null(data)) stop("Requires parameter data to give name of the real data\n", call. = FALSE)
- if (!is.data.frame(data)) stop("Argument data must be a data frame\n", call. = FALSE)          
- #if (class(object)! = "synds") stop("Object must have class synds\n", call. = FALSE )                                                                    
- if (!is.null(msel) & !all(msel %in% (1:object$m))) stop("Invalid synthesis number(s)", call. = FALSE)                                                                        
-
+ if (is.null(data)) stop("Requires parameter data to give name of the real data.\n", call. = FALSE)
+ if (!is.data.frame(data)) stop("Argument data must be a data frame.\n", call. = FALSE)          
+ if (class(object) != "synds") stop("Object must have class synds.\n", call. = FALSE )                                                                    
+ if (!is.null(msel) & !all(msel %in% (1:object$m))) stop("Invalid synthesis number(s).", call. = FALSE)
+ if (!all(utility.stats %in% c("VW", "FT", "JSD", "SPECKS", "WMabsDD", "U", "G", "pMSE", "PO50", "MabsDD", "dBhatt",
+                              "S_VW", "S_FT", "S_JSD", "S_WMabsDD", "S_G", "S_pMSE", "df", "all"))) 
+  stop('utility.stats must be set to "all" or selected from "VW", "FT", "JSD", "SPECKS", "WMabsDD", "U", "G", "pMSE", "PO50", "MabsDD", "dBhatt", "S_VW", "S_FT", "S_JSD", "S_WMabsDD", "S_G", "S_pMSE" or "df".\n', call. = FALSE)  
+  
  if (!(length(stat) == 1 & stat %in% c("percents", "counts"))) { 
    cat('Parameter stat must be "percents" or "counts".\n' )
    stat <- "percents"
@@ -64,6 +70,15 @@ compare.synds <- function(object, data, vars = NULL,
    df.syn <- vector("list",length(msel))
    for (i in 1:length(msel)) df.syn[[i]] <- synds[[i]][, commonnames, drop = FALSE]
  }
+ 
+ # change any numeric variables with < 6 distinct values to factors
+ for (i in 1:length(commonnames)) {
+   if (is.numeric(df.obs[,i]) && length(table(df.obs[,i])) < 6){
+     df.obs[,i] <- as.factor(df.obs[,i])
+     df.synall[,i] <- as.factor(df.synall[,i])
+   }
+ }
+
  num <- sapply(df.synall, is.numeric) | sapply(df.synall, is.integer)  
  fac <- sapply(df.synall, function(x) is.factor(x) | is.logical(x))  
 
@@ -75,14 +90,14 @@ compare.synds <- function(object, data, vars = NULL,
      per.syn.facall <- ggfac(df.synall[, fac, drop = FALSE], 
      name = "synthetic", anyna = any.fac.na, stat = stat)
      if (stat == "counts") {
-       per.syn.facall$perdf$Count <- per.syn.facall$perdf$Count/object$m        #BN:16/08/2018
-       per.syn.facall$perlist <- lapply(per.syn.facall$perlist,"/",object$m)    #BN:16/08/2018
+       per.syn.facall$perdf$Count <- per.syn.facall$perdf$Count/object$m
+       per.syn.facall$perlist <- lapply(per.syn.facall$perlist,"/",object$m)
      }
    }
    if (length(msel) > 1) {
      per.syn.fac <- vector("list",length(msel))
      for (i in 1:length(msel)) per.syn.fac[[i]] <- ggfac(df.syn[[i]][, fac, drop = FALSE], 
-       name = paste0("syn=", msel[i]), anyna = any.fac.na, stat = stat)   
+       name = paste0("syn=", msel[i]), anyna = any.fac.na, stat = stat)  
    }
  } else {                                       
    per.obs.fac    <- NULL
@@ -92,11 +107,12 @@ compare.synds <- function(object, data, vars = NULL,
  # frequency tables for numeric variables
  if (sum(num) > 0) {
    cont.index <- match(colnames(df.obs[, num, drop = FALSE]), colnames(syndsall))    
-   na <- object$cont.na[cont.index]  
+   na <- object$cont.na[cont.index] 
   # to exclude from summaries if no missing in data
    any.na <- unlist(apply(df.obs[, num, drop = FALSE], 2, function(x) any(is.na(x))))  
-   
-   lbreaks    <- as.list(rep(breaks, length(na)))                   
+
+   lbreaks    <- as.list(rep(breaks, length(na)))  
+
   ## get limits(=breaks) from both observed and synthetic
   #--
    df.both  <- rbind.data.frame(df.obs, df.synall)                
@@ -111,8 +127,8 @@ compare.synds <- function(object, data, vars = NULL,
        name = "synthetic", na = na, breaks = per.both$hbreaks, 
        anyna = any.na, stat = stat) 
        if (stat == "counts") {
-         per.syn.numall$perdf$Count <- per.syn.numall$perdf$Count/object$m        #BN:16/08/2018
-         per.syn.numall$perlist <- lapply(per.syn.numall$perlist,"/",object$m)    #BN:16/08/2018
+         per.syn.numall$perdf$Count <- per.syn.numall$perdf$Count/object$m
+         per.syn.numall$perlist <- lapply(per.syn.numall$perlist,"/",object$m)
        }
    } 
    if (length(msel) > 1) {
@@ -166,7 +182,7 @@ compare.synds <- function(object, data, vars = NULL,
  } else {
    for (i in 1:length(os.table)) dimnames(os.table[[i]])[[1]] <- c("observed",paste0("syn=", msel))
  }
- Value <- Percent <- Count <- Data <- NULL                               ## otherwise 'no visible binding for global variables'
+ Value <- Percent <- Count <- Data <- NULL    ## otherwise 'no visible binding for global variables'
  # sorts the factor labels in the right order for numeric vars
  per.fac$Value <- as.character(per.fac$Value)
  vals <- unique(per.fac$Value)
@@ -205,7 +221,7 @@ compare.synds <- function(object, data, vars = NULL,
               legend.position = "top", 
               legend.key = element_rect(colour = cols[1])) 
    p <- p + theme(legend.title = element_blank())
-   if (length(msel) > 1) p <- p + scale_fill_manual(values = c(cols[1],rep(cols[2],length(msel))))
+   if (length(msel) > 1) p <- p + scale_fill_manual(values = c(cols[1], rep(cols[2], length(msel))))
    if (length(msel) <= 1) p <- p + scale_fill_manual(values = cols)
    plots[[i]] <- p
  }
@@ -214,13 +230,81 @@ compare.synds <- function(object, data, vars = NULL,
    tables <- tables[[1]]
    plots  <- plots[[1]]
  }  
- #browser()
- res <- list(tables = tables, plots = plots, stat = stat) ##GR stat added
+
+ if (is.null(vars)) {
+   if (object$m == 1) vars <- names(object$syn)
+   else vars <- names(object$syn[[1]])
+ }
+ 
+ if (!is.null(utility.stats)) {
+   utility.list <- as.list(1:length(vars))
+   names(utility.list) <- vars
+   if (utility.stats[1] == "all") utility.stats <- 
+     c("VW", "FT", "JSD", "SPECKS", "WMabsDD", "U", "G", "pMSE", "PO50", 
+       "MabsDD", "dBhatt","S_VW", "S_FT", "S_JSD", "S_WMabsDD", "S_G", 
+       "S_pMSE", "df")
+   tab.utility <- matrix(NA, length(vars), length(utility.stats))
+   dimnames(tab.utility) <- list(vars, utility.stats)
+   
+   for (i in 1:length(vars)) {
+     utility.list[[i]] <- utility.tab(object, data, vars = vars[i])
+       if (i == 1) tab.ind <- match(utility.stats, names(utility.list[[i]]))
+       tab.utility[i, ] <- sapply(utility.list[[i]][tab.ind], mean)
+   } 
+ } else {
+   tab.utility <- NULL
+ }
+
+ res <- list(tables = tables, plots = plots, stat = stat, vars = vars,
+             tab.utility = tab.utility, table = table, plot = plot)
+
  class(res) <- "compare.synds"
  return(res)
 }
 
 
+###-----compare.data.frame---compare.list----------------------------------    
+compare.data.frame <- compare.list <- function(object, data, vars = NULL, cont.na = NULL,         
+                                               msel = NULL, stat = "percents", breaks = 20, 
+                                               nrow = 2, ncol = 2, rel.size.x = 1,
+                                               utility.stats = c("pMSE", "S_pMSE", "df"),
+                                               cols = c("#1A3C5A","#4187BF"),   
+                                               plot = TRUE, table = FALSE, ...){
+  
+  if (is.null(data)) stop("Requires parameter 'data' to give name of the real data.\n\n",  call. = FALSE)
+  if (is.null(object)) stop("Requires parameter 'object' to give name of the synthetic data.\n\n",  call. = FALSE)   
+  
+  if (is.list(object) & !is.data.frame(object)) m <- length(object)
+  else if (is.data.frame(object)) m <- 1
+  else stop("object must be a data frame or a list of data frames.\n", call. = FALSE)
+ 
+  # sort out cont.na to make it into a complete named list
+  cna <- cont.na
+  cont.na <- as.list(rep(NA, length(data)))
+  names(cont.na) <- names(data)
+  if (!is.null(cna)) {
+    if (!is.list(cna) | any(names(cna) == "") | is.null(names(cna))) 
+      stop("Argument 'cont.na' must be a named list with names of selected variables.", call. = FALSE)  
+    if (any(!names(cna) %in% names(data))) stop("Names of the list cont.na must be variables in data.\n", call. = FALSE)
+    for (i in 1:length(cna)) {
+      j <- (1:length(data))[names(cna)[i] == names(data)]
+      cont.na[[j]] <- unique(c(NA,cna[[i]]))
+    }
+  }
+
+  object <- list(syn = object, m = m, cont.na = cont.na)
+  class(object ) <- "synds"
+  
+  res <- compare.synds(object = object, data = data, vars = vars, 
+                       msel = msel, stat = stat, breaks = breaks, 
+                       nrow = nrow, ncol = ncol, rel.size.x = rel.size.x,
+                       utility.stats = utility.stats,
+                       cols = cols, plot = plot, table = table, ...) 
+  res$call <- match.call()
+  return(res)
+}
+
+
 ###-----pertable-----------------------------------------------------------
 # calculate counts/percentages
 pertable <- function(x, stat, ...) { ##GR stat added
@@ -406,13 +490,13 @@ compare.fit.synds <- function(object, data, plot = "Z",
        QB[i,] <- object$mcoef[i,] - object$mcoefavg
      }
      lof.varcov <- t(QB) %*% QB/(m - 1)/m
-     lack.of.fit <- t(res.diff[,3]) %*% solve(lof.varcov) %*% res.diff[,3]
+     lack.of.fit <- t(res.diff[,3]) %*% ginv(lof.varcov) %*% res.diff[,3]
      lack.of.fit <- lack.of.fit * (object$m - ncoef)/ncoef/(object$m - 1) # Hotellings T square
      lof.pvalue  <- 1 - pf(lack.of.fit, ncoef, object$m - ncoef)          
    }
  } else {
    lof.varcov <- real.varcov*n/k/m
-   lack.of.fit <- t(res.diff[,3]) %*% solve(lof.varcov) %*% res.diff[,3]  #! multiply by m for combined test
+   lack.of.fit <- t(res.diff[,3]) %*% ginv(lof.varcov) %*% res.diff[,3]  #! multiply by m for combined test
    lof.pvalue  <- 1 - pchisq(lack.of.fit, ncoef)
  }
 
@@ -486,7 +570,7 @@ compare.fit.synds <- function(object, data, plot = "Z",
    lack.of.fit = lack.of.fit, lof.pvalue = lof.pvalue, 
    ci.plot = p, print.coef = print.coef,       
    m = object$m, ncoef = ncoef,
-   incomplete = incomplete, 
+   incomplete = incomplete,
    population.inference = population.inference)  
 
  class(res) <- "compare.fit.synds"
