@@ -26,7 +26,8 @@ print.synds <- function(x, ...){
 
 summary.synds <- function(object, msel = NULL, 
   maxsum = 7, digits = max(3, getOption("digits") - 3), ...){
-  if (!is.null(msel) & !all(msel %in% (1:object$m))) stop("Invalid synthesis number(s)", call. = FALSE)
+  if (!is.null(msel) & !all(msel %in% (1:object$m))) 
+    stop("Invalid synthesis number(s)", call. = FALSE)
 
   sy <- list(m = object$m, msel = msel, method = object$method)
 
@@ -56,7 +57,7 @@ summary.synds <- function(object, msel = NULL,
 print.summary.synds <- function(x, ...){
 
  if (x$m == 1) {
-   cat("Synthetic object with one synethesis using methods:\n")
+   cat("Synthetic object with one synthesis using methods:\n")
    print(x$method)
    cat("\n")
    print(x$result)
@@ -116,7 +117,7 @@ mcoefvar <- function(analyses, ...) {
 
 lm.synds <- function(formula, data, ...)
 {
- if (!class(data) == "synds") stop("Data must have class synds\n")
+ if (!class(data) == "synds") stop("Data must have class synds\n", call. = FALSE)
  if (is.matrix(data$method)) data$method <- data$method[1,]
  if (is.matrix(data$visit.sequence)) data$visit.sequence <- data$visit.sequence[1,]
  if (data$m > 1) vars <- names(data$syn[[1]])  else  vars <- names(data$syn)  
@@ -158,7 +159,7 @@ lm.synds <- function(formula, data, ...)
 
 glm.synds <- function(formula, family = "binomial", data, ...)
 {
- if (!class(data) == "synds") stop("Data must have class synds\n")
+ if (!class(data) == "synds") stop("Data must have class synds\n", call. = FALSE)
  if (is.matrix(data$method)) data$method <- data$method[1,]
  if (is.matrix(data$visit.sequence)) data$visit.sequence <- data$visit.sequence[1,]
  if (data$m > 1) vars <- names(data$syn[[1]])  else  vars <- names(data$syn)  
@@ -200,7 +201,7 @@ glm.synds <- function(formula, family = "binomial", data, ...)
 
 polr.synds <- function(formula, data, ...)
 {
-  if (!class(data) == "synds") stop("Data must have class 'synds'.\n")
+  if (!class(data) == "synds") stop("Data must have class 'synds'.\n", call. = FALSE)
   if (is.matrix(data$method)) data$method <- data$method[1,]
   if (is.matrix(data$visit.sequence)) data$visit.sequence <- data$visit.sequence[1,]
   if (data$m > 1) vars <- names(data$syn[[1]]) else  vars <- names(data$syn)  
@@ -241,7 +242,7 @@ polr.synds <- function(formula, data, ...)
 
 multinom.synds <- function(formula, data, ...)
 {
-  if (!class(data) == "synds") stop("Data must have class 'synds'.\n")
+  if (!class(data) == "synds") stop("Data must have class 'synds'.\n", call. = FALSE)
   if (is.matrix(data$method)) data$method <- data$method[1,]
   if (is.matrix(data$visit.sequence)) data$visit.sequence <- data$visit.sequence[1,]
   if (data$m > 1) vars <- names(data$syn[[1]]) else  vars <- names(data$syn)  
@@ -352,7 +353,7 @@ print.fit.synds <- function(x, msel = NULL, ...)
 summary.fit.synds <- function(object, population.inference = FALSE, msel = NULL, 
                               real.varcov = NULL, ...)
 { # df.residual changed to df[2] because didn't work for lm 
-  if (!class(object) == "fit.synds") stop("Object must have class fit.synds\n")
+  if (!class(object) == "fit.synds") stop("Object must have class fit.synds\n", call. = FALSE)
   m <- object$m
   n <- object$n
   k <- object$k
@@ -492,7 +493,7 @@ print.compare.fit.synds <- function(x, print.coef = x$print.coef, ...){
     cat("\nCombined estimates for the synthesised data set(s):\n")
     print(x$coef.syn)
   }  
-    
+
   cat("\nDifferences between results based on synthetic and observed data:\n")
     print(cbind.data.frame(x$coef.diff,x$ci.overlap))                                 
   if (x$m == 1) {
@@ -503,9 +504,14 @@ print.compare.fit.synds <- function(x, print.coef = x$print.coef, ...){
   cat("\nMean confidence interval overlap: ", x$mean.ci.overlap)
   cat("\nMean absolute std. coef diff: ", x$mean.abs.std.diff)
   if (!is.null(x$lack.of.fit)){
-    cat("\nLack-of-fit: ", x$lack.of.fit,"; p-value ", round(x$lof.pval,3), " for test that synthesis model is compatible ", sep = "")
-    if (x$incomplete == FALSE) cat("\nwith a chi-squared test with ", x$ncoef, " degrees of freedom\n", sep = "")
-    else cat("\nwith an F distribution with ",x$ncoef," and ",x$m - x$ncoef," degrees of freedom\n", sep = "") 
+    cat("\n\nMahalanobis distance ratio for lack-of-fit (target 1.0): ", 
+        round(x$lack.of.fit/x$ncoef, 2), sep = "")
+    cat("\nLack-of-fit test: ", x$lack.of.fit,"; p-value ", round(x$lof.pval, 4), 
+        " for test that synthesis model is compatible ", sep = "")
+    if (x$incomplete == FALSE) cat("\nwith a chi-squared test with ", 
+                                   x$ncoef, " degrees of freedom.\n", sep = "")
+    else cat("\nwith an F distribution with ", x$ncoef, " and ", 
+             x$m - x$ncoef, " degrees of freedom.\n", sep = "") 
   }
   if (!is.null(x$ci.plot)) {
     cat("\nConfidence interval plot:\n")
@@ -521,80 +527,109 @@ print.compare.synds <- function(x, ...) {
   if (x$stat == "counts") cat("\nComparing counts observed with synthetic\n\n") 
   else cat("\nComparing percentages observed with synthetic\n\n")
   if (class(x$plots)[1] == "gg") {
-    print(x$tables) 
-    print(x$plots)
+    if (x$table) print(x$tables) 
+    if (x$plot) print(x$plots)
   } else {
     for (i in 1:length(x$tables)) {
-      print(x$tables[[i]]) 
-      print(x$plots[[i]])
-      if (i < length(x$tables)) {
-        cat("Press return for next plot: ")
-        ans <- readline()
+      if (x$table) print(x$tables[[i]]) 
+      if (x$plot) {
+        print(x$plots[[i]])
+        if (i < length(x$tables)) {
+          cat("Press return for next variable(s): ")
+          ans <- readline()
+        }
       }
     }
   }
+  if (!is.null(x$tab.utility)) {
+    cat("\nSelected utility measures:\n")  
+    print(round(x$tab.utility, 6))
+  }
+  #if (x$print.tab.utility) print(x$tab.utility)
  invisible(x)
 }
 
 
 ###-----print.utility.gen--------------------------------------------------
 
-print.utility.gen <- function(x, digits = x$digits,  
-  print.zscores = x$print.zscores, zthresh = x$zthresh, 
-  print.ind.results = x$print.ind.results,
-  print.variable.importance = x$print.variable.importance, ...){
+print.utility.gen <- function(x, digits = NULL, zthresh = NULL,    
+                              print.zscores = NULL, print.stats = NULL,
+                              print.ind.results = NULL,                               
+                              print.variable.importance = NULL, ...){
+
+  if (x$method == "logit"){  
+    converged <- ifelse(x$m == 1, x$fit$converged, x$fit[[1]]$converged)
+    length.y <- ifelse(x$m == 1, length(x$fit$y)/2, length(x$fit[[1]]$y)/2)
+    length.coef <- ifelse(x$m == 1, length(x$fit$coefficients), length(x$fit[[1]]$coefficients)) 
+    if (converged == FALSE)  cat("Warning: Logistic model did not converge in ", x$maxit,
+    " iterations. Check results.\nThis can be due to cells with small numbers or too complicated a model.
+In first case utility measures are probably fine but coefficients may be unstable. 
+In the latter case you may be able to overcome this by increasing the parameter 'maxit'.
+Your combined data has ", length.y, " observations and your model has ",
+    length.coef, " parameters.\n", sep = "")
+  }
   
-  cat("\nUtility score calculated by method: ", x$method, "\n")
+  if (is.null(digits)) digits <- x$digits
+  if (is.null(print.stats)) print.stats <- x$print.stats
+  if (is.null(zthresh)) zthresh <- x$zthresh
+  if (is.null(print.zscores)) print.zscores <- x$print.zscores
+  if (is.null(print.ind.results)) print.ind.results <- x$print.ind.results   
+  if (is.null(print.variable.importance)) print.variable.importance <- x$print.variable.importance
+  
+  cat("\nUtility score calculated by method: ", x$method, "\n", sep = "")
   cat("\nCall:\n")
   print(x$call)
-  
-  if (!is.null(x$resamp.method)) {
-    if (x$resamp.method == "perm") cat("\nNull utility simulated from a permutation test with ", x$nperm," replications\n", sep = "")
-    else if (x$resamp.method == "pairs") cat("\nNull utility simulated from ", x$m*(x$m - 1)/2," pairs of syntheses\n", sep = "")
+#?  
+  if (!is.null(x$resamp.method) && !x$resamp.method == "none") {
+    if (x$resamp.method == "perm") cat("\nNull utilities simulated from a permutation test with ", x$nperm," replications.\n", sep = "")
+    else if (x$resamp.method == "pairs") cat("\nNull utilities simulated from ", x$m*(x$m - 1)/2," pair(s) of syntheses.\n", sep = "")
     
     if (!is.list(x$nnosplits)) { 
       if (x$nnosplits[1] > 0) cat(
 "\n***************************************************************
-Warning: null utility resamples failed to split ", x$nnosplits[1], " times from ", x$nnosplits[2],
+Warning: null pMSE resamples failed to split ", x$nnosplits[1], " time(s) from ", x$nnosplits[2],
 "\n***************************************************************\n", sep = "")
     } else {
       for (ss in 1:x$m) {
         if (x$nnosplits[[ss]][1] > 0) cat("\nSynthesis ", ss, 
-          " null utility resamples failed to split ", x$nnosplits[[ss]][1],
-          " times from ", x$nnosplits[[ss]][2], sep = "")
+          "   null pMSE resamples failed to split ", x$nnosplits[[ss]][1],
+          " time(s) from ", x$nnosplits[[ss]][2], sep = "")
       }
       cat("\n")
     }
   }
+
+  if (print.stats[1] == "all") print.stats <- 
+    c("pMSE", "S_pMSE", "SPECKS", "S_SPECKS", "PO50", "S_PO50", "U", "S_U")
   
-  if (x$m > 1) {
-    cat("\nMean utility score results from ", x$m, " syntheses\n",
-        "Utility: ", round(mean(x$utilVal), digits),
-        "\nExpected value: ", round(mean(x$utilExp), digits),
-        "\nRatio to expected: ", round(mean(x$utilR), digits),
-        "\npMSE (propensity score mean square error): ", round(mean(x$pMSE),digits), "\n\n", sep = "")
-    
+  if (x$m == 1) { 
+    cat("\nSelected utility measures\n")  
+    tabres <- c(round(x$pMSE, digits), round(x$S_pMSE, digits), 
+                round(x$SPECKS, digits), round(x$S_SPECKS, digits),
+                round(x$PO50, digits), round(x$S_PO50, digits),
+                round(x$U, digits), round(x$S_U, digits))
+    names(tabres) <- c("pMSE", "S_pMSE", "SPECKS", "S_SPECKS", "PO50", "S_PO50", "U", "S_U")
+    tabres <- tabres[match(print.stats, names(tabres))]
+    print(tabres)
+  }
+  else {
+    cat("\nMean utility results from ", x$m, " syntheses:\n", sep = "")
+    tabres <- data.frame(round(x$pMSE, digits), round(x$S_pMSE, digits), 
+                round(x$SPECKS, digits), round(x$S_SPECKS, digits),
+                round(x$PO50, digits), round(x$S_PO50, digits),
+                round(x$U, digits), round(x$S_U, digits))
+    names(tabres) <- c("pMSE", "S_pMSE", "SPECKS", "S_SPECKS", "PO50", "S_PO50", "U", "S_U")
+    tabres <- tabres[, match(print.stats, names(tabres)), drop = FALSE]
+    print(round(apply(tabres, 2, mean), digits))
     if (print.ind.results == TRUE) {
-      cat("Individual utility score results from ", x$m, " syntheses\n", sep = "")
-      tabres <- data.frame(round(x$utilVal, digits), round(x$utilExp, digits), 
-                           round(x$utilR, digits), x$pval, round(x$pMSE, digits))
-      names(tabres) <- c("Utility", "Expected", "Ratio", "p-value", "pMSE")
+      cat("\nIndividual utility results from ", x$m, " syntheses:\n", sep = "")
       print(tabres)
-      cat("\n")
     }
-    
-  } else {
-    cat("\nUtility score results",
-        "\nUtility score: ", round(x$utilVal, digits),
-        "\nExpected value: ", round(x$utilExp, digits),
-        "\nRatio to expected: ", round(x$utilR, digits),
-        "\np-value: ", x$pval,
-        "\npMSE (propensity score mean square error): ", round(x$pMSE, digits), "\n\n", sep = "")
   }
   
   if (print.zscores == TRUE) {
     if (x$method == "cart") {
-      cat("\nz-scores not available for CART models\n")
+      cat("\nz-scores are not available for CART models.\n")
     } else {
       if (x$m > 1) {
         allzscores <- vector("list", x$m) 
@@ -605,7 +640,7 @@ Warning: null utility resamples failed to split ", x$nnosplits[1], " times from
         zscores <- apply(allzscores.NA.df, 1, mean, na.rm = TRUE)  
         names(zscores) <- allnames
       } else {
-        zscores <- summary(x$fit)$coefficients[ ,3]
+        zscores <- summary(x$fit)$coefficients[, 3]
       }
       
       if (!is.na(zthresh)) { 
@@ -614,24 +649,24 @@ Warning: null utility resamples failed to split ", x$nnosplits[1], " times from
           cat("\nNo z-scores (or mean z-scores if m > 1) above threshold of +/-", zthresh,"\n", sep = "")
         } else {
           cat("\nz-scores (or mean z-scores if m > 1) greater than the threshold of +/- ", zthresh, "\n", sep = "")
-          print(zscores)
+          print(round(zscores, digits))
         }  
       } else {
         cat("\nAll z-scores (or mean z-scores if m > 1)\n")
-        print(zscores)
+        print(round(zscores, digits))
       }
     }
   }
-  #browser()
+  
   if (print.variable.importance == TRUE) {
     if (x$method != "cart" | x$tree.method != "rpart") {
-      cat("\nVariable importance only available for CART models using function 'rpart'\n")
+      cat("\nVariable importance only available for CART models using function 'rpart' (tree.method).\n")
     } else {
       cat("\nRelative importance of each variable scaled to add to 100\n" )
       if (x$m == 1) {
         variable.importance <- x$fit$variable.importance
         variable.importance <- round(variable.importance/sum(variable.importance)*100, digits)
-        print(variable.importance)
+        print(round(variable.importance), digits)
       } else {
         cat("(results for ", x$m, " syntheses)\n", sep = "")
         variable.importance <- vector("list", x$m)
@@ -645,7 +680,7 @@ Warning: null utility resamples failed to split ", x$nnosplits[1], " times from
         all.vars.importance <- do.call(rbind, all.vars)
         colnames(all.vars.importance) <- allnames
         rownames(all.vars.importance) <- 1:x$m
-        print(all.vars.importance)
+        print(round(all.vars.importance, digits))
       }
     }
   }
@@ -656,84 +691,67 @@ Warning: null utility resamples failed to split ", x$nnosplits[1], " times from
 
 ###-----print.utility.tab--------------------------------------------------
 
-print.utility.tab <- function(x, print.tables = x$print.tables,  
-                              print.zdiff = x$print.zdiff, 
-                              print.stats = x$print.stats, 
-                              digits = x$digits, ...){
+print.utility.tab <- function(x, print.tables = NULL,
+                              print.zdiff = NULL, 
+                              print.stats = NULL, 
+                              digits = NULL, ...){
+  
+  if (is.null(print.stats)) print.stats <- x$print.stats
+  if (is.null(print.zdiff)) print.zdiff <- x$print.zdiff
+  if (is.null(print.tables)) print.tables <- x$print.tables
+  if (is.null(digits)) digits <- x$digits
   
   if (print.tables == TRUE) {
-    if (is.table(x$tab.obs)) {
-      if (sum(x$tab.obs) != x$n) { 
-        cat("\nObserved adjusted to match the size of the synthetic data: \n($tab.obs)\n")
-        print(round(x$tab.obs, digits))
-      } else {
-        cat("\nObserved: \n($tab.obs)\n")
-        print(x$tab.obs)
-      }  
+    if (x$k.syn) cat("\nObserved not adjusted to match the size of the synthetic data since 'k.syn' set to TRUE:\n($tab.obs)\n")
+    else if (sum(x$tab.obs) != x$n & !x$k.syn) { 
+      cat("\nObserved adjusted to match the size of the synthetic data:\n($tab.obs)\n")
+      print(round(x$tab.obs, digits))
     } else {
-      #if (sum(x$tabd)/length(x$tabd)!= x$n) 
-      cat("\nMean of ",x$m," observed tables ($tab.obs) adjusted to match the size of synthetic data:\n", sep = "")
-      meantabd <- apply(simplify2array(x$tab.obs), c(1,2), mean)
-      print(round(meantabd, digits))
-    } 
-    
+      cat("\nObserved:\n($tab.obs)\n")
+      print(x$tab.obs)
+    }  
+  
     if (x$m == 1) {
       cat("\nSynthesised: \n($tab.syn)\n")
       print(x$tab.syn) 
     } else {
-      meantab <- apply(simplify2array(x$tab.syn), c(1,2), mean)
-      cat("\nMean of ",x$m," synthetic tables ($tab.syn):\n", sep = "")
+      if (length(dim(simplify2array(x$tab.syn))) == 2) {
+        meantab <- apply(simplify2array(x$tab.syn), 1, mean)
+      } else {
+        meantab <- apply(simplify2array(x$tab.syn), c(1, 2), mean)
+      }
+      cat("\nMean of ", x$m, " synthetic tables ($tab.syn):\n", sep = "")
       print(round(meantab, digits))
     }
   }
   
   if (print.zdiff == TRUE) {
-    cat("\nTable of z-scores for differences: \n($tab.zdiff)\n")
     if (x$m == 1) {
+      cat("\nTable of z-scores for differences: \n($tab.zdiff)\n")      
       print(round(x$tab.zdiff, digits)) 
     } else {
-      meanzdiff <- apply(simplify2array(x$tab.zdiff), c(1,2), mean)
-      cat("\nMean of ",x$m," z-score tables:\n", sep = "")
+      if (length(dim(simplify2array(x$tab.zdiff))) == 2) {
+        meanzdiff <- apply(simplify2array(x$tab.zdiff), 1, mean)
+      } else {
+        meanzdiff <- apply(simplify2array(x$tab.zdiff), c(1, 2), mean)
+      }
+      cat("\nMean of ", x$m, " z-score tables for differences ($tab.zdiff):\n", sep = "")
       print(round(as.table(meanzdiff), digits))
     }
   }
-  
-  if (x$m == 1) {
-    cat("\nNumber of cells in each table: ", 
-        x$df[1] + x$nempty[1] + 1,
-        "; Number of cells contributing to utility measures: ", 
-        x$df + 1,"\n", sep = "")
-    cat("\nUtility score results\n")
-    if ("VW" %in% print.stats) cat("Voas Williamson (VW): ", round(x$UtabVW,digits), ";",
-      " Ratio to degrees of freedom (df): ", round(x$ratioVW,digits), ";",
-      " p-value: ", x$pvalVW, "\n", sep = "")
-    if ("FT" %in% print.stats) cat("Freeman Tukey (FT): ", round(x$UtabFT,digits), ";",
-      " Ratio to degrees of freedom (df): ", round(x$ratioFT,digits), ";",
-      " p-value: ", x$pvalFT, "\n", sep = "")
-  } else if (x$m > 1) {
-    cat("\nAverage results for ", x$m, " syntheses\n", sep = "")
-    cat("\nNumber of cells in each table: ", 
-        round(mean(x$df[1] + x$nempty[1] + 1), digits),
-        "; Number of cells contributing to utility measures: ", 
-        round(mean(x$df + 1), digits),"\n", sep = "")
-    cat("\nAverage utility score results\n")
-    if ("VW" %in% print.stats) cat("Voas Williamson (VW): ", round(mean(x$UtabVW), digits), ";",
-      " Ratio to degrees of freedom (df): ",  round(mean(x$ratioVW), digits),"\n", sep = "")    
-    if ("FT" %in% print.stats) cat("Freeman Tukey (FT): ", round(mean(x$UtabFT),digits), ";",
-      " Ratio to degrees of freedom (df): ", round(mean(x$ratioFT),digits),"\n", sep = "")
+  if (print.stats[1] == "all") print.stats <- c("VW", "FT", "JSD", 
+    "SPECKS", "WMabsDD", "U", "G", "pMSE", "PO50", "MabsDD", "dBhatt",
+    "S_VW", "S_FT", "S_JSD", "S_WMabsDD", "S_G", "S_pMSE", "df", "dfG")
 
-    cat("\nResults from individual syntheses\n")
-    tab.res <- cbind.data.frame(x$df,
-                                round(x$UtabVW,digits), round(x$pvalVW,digits),
-                                round(x$UtabFT,digits), round(x$pvalFT,digits))
-    colnames(tab.res) <- c("df", 
-                           "VW Utility", "VW p-value",
-                           "FT Utility", "FT p-value")
-    if (!("VW" %in% print.stats)) tab.res <- tab.res[,-(2:3)]
-    if (!("FT" %in% print.stats)) tab.res <- tab.res[,-(4:5)]
-    print(tab.res)
-  }
-  
+  if (x$k.syn) celldiff <- 0 else celldiff <- 1
+  tab.res <- matrix(NA, x$m, length(print.stats))
+  dimnames(tab.res) <- list(1:x$m, print.stats)
+  if (x$m > 1) cat("\nSelected utility measures from ", x$m, " syntheses:\n", sep = "")
+  else cat("\nSelected utility measures:\n")
+  for (i in 1:length(print.stats)) {
+    tab.res[, i] <- unlist(x[match(print.stats[i], names(x))])
+  } 
+  print(round(tab.res, digits))
   invisible(x)
 }
 
@@ -787,7 +805,7 @@ summary.out <- function(z, digits = max(3L, getOption("digits") - 3L), ...)
         z <- unlist(z, use.names = TRUE)
         dim(z) <- c(nr, nv)
         if (anyNA(lw))
-            warning("probably wrong encoding in names(.) of column ",
+            warning("Probably wrong encoding in names(.) of column ",
                 paste(which(is.na(lw)), collapse = ", "))
         blanks <- paste(character(max(lw, na.rm = TRUE) + 2L),
             collapse = " ")
